/* 
 * tkWinInit.c --
 *
 *	This file contains Windows-specific interpreter initialization
 *	functions.
 *
 * Copyright (c) 1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * SCCS: @(#) tkWinInit.c 1.12 96/03/08 13:46:49
 */

#include "tkWinInt.h"


/*
 *----------------------------------------------------------------------
 *
 * TkPlatformInit --
 *
 *	Performs Windows-specific interpreter initialization related to the
 *      tk_library variable.
 *
 * Results:
 *	A standard Tcl completion code (TCL_OK or TCL_ERROR).  Also
 *	leaves information in interp->result.
 *
 * Side effects:
 *	Sets "tk_library" Tcl variable, runs "tk.tcl" script.
 *
 *----------------------------------------------------------------------
 */

int
TkPlatformInit(interp)
    Tcl_Interp *interp;
{
    char *ptr;
    HKEY regKey;
    DWORD size = MAX_PATH;
    DWORD result;
    OSVERSIONINFO info;
    int isWin32s;		/* True if we are running under Win32s. */
    Tcl_DString ds, temp;
    static char initCmd[] =
	"if [file exists $tk_library/tk.tcl] {\n\
	    source $tk_library/tk.tcl\n\
	} else {\n\
	    set msg \"can't find $tk_library/tk.tcl; perhaps you \"\n\
	    append msg \"need to\\ninstall Tk or set your TK_LIBRARY \"\n\
	    append msg \"environment variable?\"\n\
	    error $msg\n\
	}";

    Tcl_DStringInit(&ds);

    /*
     * Find out if we're running on Win32s
     */

    info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    GetVersionEx(&info);
    isWin32s = (info.dwPlatformId == VER_PLATFORM_WIN32s);

    /*
     * Look in the environment for a TK_LIBRARY entry.
     * Store the result in the dynamic string.
     */

    ptr = Tcl_GetVar2(interp, "env", "TK_LIBRARY", TCL_GLOBAL_ONLY);
    Tcl_DStringSetLength(&ds, 0);
    if (ptr) {
	Tcl_DStringAppend(&ds, ptr, -1);
    }

    /*
     * If the path doesn't exist, look for the library relative to the
     * tk.dll library.
     */

    if ((Tcl_DStringLength(&ds) == 0)
	    || (access(Tcl_DStringValue(&ds), 0) != 0)) {
	char *lastSlash;

	/*
	 * Path was invalid, so we need to construct a default path from the
	 * location of the executable.  Convert backslashes at the same time.
	 */

	Tcl_DStringSetLength(&ds, MAX_PATH);
	Tcl_DStringSetLength(&ds, GetModuleFileName(TkWinGetTkModule(),
		Tcl_DStringValue(&ds), MAX_PATH));
	if (Tcl_DStringLength(&ds) > 0) {
	    lastSlash = strrchr(Tcl_DStringValue(&ds), '\\');
	    Tcl_DStringSetLength(&ds, lastSlash - Tcl_DStringValue(&ds));
	    Tcl_DStringAppend(&ds, "\\..\\lib\\tk", -1);
	    Tcl_DStringAppend(&ds, TK_VERSION, -1);
	}
    }

    ptr = TclConvertToNetwork(NULL, Tcl_DStringValue(&ds), &temp);
    if (!ptr) {
	ptr = "";
    }
    Tcl_SetVar(interp, "tk_library", ptr, TCL_GLOBAL_ONLY);
    Tcl_DStringFree(&temp);
    Tcl_DStringFree(&ds);

    return Tcl_Eval(interp, initCmd);
}
